"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var apputils_1 = require("@jupyterlab/apputils");
var coreutils_1 = require("@jupyterlab/coreutils");
var virtualdom_1 = require("@phosphor/virtualdom");
/**
 * Prompt the user what do about companion packages, if present
 *
 * @param builder the build manager
 */
function presentCompanions(kernelCompanions, serviceManager) {
    if (kernelCompanions) {
        var entries = [];
        for (var _i = 0, kernelCompanions_1 = kernelCompanions; _i < kernelCompanions_1.length; _i++) {
            var entry = kernelCompanions_1[_i];
            entries.push(virtualdom_1.h.p('The package ', virtualdom_1.h.code(entry.kernelInfo.base.name), ' is required by the following kernels:'));
            var kernelEntries = [];
            for (var _a = 0, _b = entry.kernels; _a < _b.length; _a++) {
                var kernel = _b[_a];
                kernelEntries.push(virtualdom_1.h.li(virtualdom_1.h.code(kernel.display_name)));
            }
            entries.push(virtualdom_1.h.ul.apply(virtualdom_1.h, kernelEntries));
        }
        var body = virtualdom_1.h.div.apply(virtualdom_1.h, [virtualdom_1.h.p('This package has indicated that it needs a corresponding package for the kernel.')].concat(entries));
        return apputils_1.showDialog({
            title: 'Kernel companions',
            body: body,
            buttons: [
                apputils_1.Dialog.cancelButton(),
                apputils_1.Dialog.warnButton({
                    label: 'Install in Kernel',
                    caption: 'Try to install the package into the selected kernels.'
                }),
                apputils_1.Dialog.okButton({
                    label: 'Install Ext Only',
                    caption: 'Install only the Jupyterlab frontend extension.'
                })
            ],
        }).then(function (result) {
            if (result.button.label === 'Install in Kernel') {
                return promptInstallCompanions(kernelCompanions, serviceManager);
            }
            return result.button.label === 'Install Ext Only';
        });
    }
    return Promise.resolve(true);
}
exports.presentCompanions = presentCompanions;
/**
 * Prompt the user what do about companion packages, if present
 *
 * @param builder the build manager
 */
function promptInstallCompanions(kernelCompanions, serviceManager) {
    // VDOM entries to put in dialog:
    var entries = [];
    // Config (model) to be filled by dialog:
    var config = {};
    var _loop_1 = function (entry) {
        var lookupName = entry.kernelInfo.base.name;
        var kernelEntries = [];
        // Create initial config:
        config[lookupName] = {
            kernelInfo: entry.kernelInfo,
            manager: entry.kernelInfo.managers[0] || '',
            selected: {},
        };
        var _loop_2 = function (kernel) {
            // For each entry, create a checkbox:
            kernelEntries.push(virtualdom_1.h.label(virtualdom_1.h.input({ type: 'checkbox', value: kernel.name, onchange: function () {
                    // Have checkbox toggle modify config:
                    var selected = config[lookupName].selected;
                    if (kernel.name in selected) {
                        delete selected[kernel.name];
                    }
                    else {
                        selected[kernel.name] = kernel;
                    }
                } }), kernel.display_name), virtualdom_1.h.br());
        };
        for (var _i = 0, _a = entry.kernels; _i < _a.length; _i++) {
            var kernel = _a[_i];
            _loop_2(kernel);
        }
        // Add select for picking which package panager to use
        var managerOptions = [];
        for (var _b = 0, _c = entry.kernelInfo.managers || []; _b < _c.length; _b++) {
            var m = _c[_b];
            managerOptions.push(virtualdom_1.h.option({ value: m }, m));
        }
        entries.push(virtualdom_1.h.div.apply(virtualdom_1.h, [entry.kernelInfo.base.name,
            virtualdom_1.h.select(managerOptions)].concat(kernelEntries)));
    };
    for (var _i = 0, kernelCompanions_2 = kernelCompanions; _i < kernelCompanions_2.length; _i++) {
        var entry = kernelCompanions_2[_i];
        _loop_1(entry);
    }
    var body = virtualdom_1.h.div.apply(virtualdom_1.h, [virtualdom_1.h.p('Which kernel do you want to install into?')].concat(entries));
    var dialogPromise = apputils_1.showDialog({
        title: 'Install kernel companions',
        body: body,
        buttons: [
            apputils_1.Dialog.cancelButton(),
            apputils_1.Dialog.warnButton({ label: 'Install' })
        ],
    });
    var installPromise = dialogPromise.then(function (result) {
        if (!result.button.accept) {
            return;
        }
        var _loop_3 = function (key) {
            var c = config[key];
            for (var _i = 0, _a = Object.keys(c.selected); _i < _a.length; _i++) {
                var kernelName = _a[_i];
                var spec = c.selected[kernelName];
                serviceManager.sessions.startNew({
                    path: coreutils_1.uuid(16),
                    kernelName: spec.name,
                }).then(function (session) {
                    var kernel = session.kernel;
                    var override = {};
                    if (c.kernelInfo.overrides && c.manager in c.kernelInfo.overrides) {
                        override = c.kernelInfo.overrides[c.manager];
                    }
                    var info = __assign({}, c.kernelInfo.base, override);
                    installOnKernel(kernel, c.manager, info).then(function () {
                        session.shutdown();
                    });
                });
            }
        };
        // Start launching kernels, build commands, and send commands
        for (var _i = 0, _a = Object.keys(config); _i < _a.length; _i++) {
            var key = _a[_i];
            _loop_3(key);
        }
    });
    return installPromise.then(function () {
        return dialogPromise;
    }).then(function (result) {
        if (!result.button.accept) {
            return false;
        }
        // Loop over all configured actions, and see if any bundle JS
        var bundled = false;
        for (var _i = 0, _a = Object.keys(config); _i < _a.length; _i++) {
            var key = _a[_i];
            var c = config[key];
            var override = {};
            if (c.kernelInfo.overrides && c.manager in c.kernelInfo.overrides) {
                override = c.kernelInfo.overrides[c.manager];
            }
            var info = __assign({}, c.kernelInfo.base, override);
            bundled = bundled || !!info.bundles_extension;
        }
        // If JS is bundled, prevent direct install of NPM package
        return !bundled;
    });
}
exports.promptInstallCompanions = promptInstallCompanions;
function installOnKernel(kernel, manager, info) {
    if (manager === 'pip') {
        var code = "\nimport sys\nfrom subprocess import check_call\ncheck_call([sys.executable, '-m', 'pip', 'install', '" + info.name + "'])\n";
        var future = kernel.requestExecute({
            code: code,
            stop_on_error: true
        });
        return future.done.then(function (reply) {
            if (reply.content.status !== 'ok') {
                console.error('Error installing on kernel', reply);
                throw new Error("Error installing on kernel:\n" + reply.content.status);
            }
        });
    }
    return Promise.reject("Unknown manager: " + manager);
}
//# sourceMappingURL=dialog.js.map